function approx = mmps_approx(idn,n1,n2,S,params)

    rng('shuffle');
 
    tic;
    
    % Numbers/Initiation ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ 

    nvar = size(S,2)+1;
    N = nvar.*(n1+n2);    
    Solutions = zeros(params.ns,N);
    obj = Inf(params.ns,1);
    flag = -ones(params.ns,1);
        
    % Generating grids/vectors ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ 
    
    if params.tsm == 0
        xp = params.modelfunc(...
            S(:,params.nx+1:end)',S(:,1:params.nx)')';
    else
        xp = params.modelfunc(...
            S(:,params.nx+1:end)',S(:,1:params.nx)')'.*params.tsm;
    end
    NL = xp(:,idn);
    weight = 1./(abs(NL)+0.001);
    lb = -Inf(n1+n2,nvar);
    ub = Inf(n1+n2,nvar);
    
    % Test (approximate) Jacobian ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ 
    
    d = randsample(size(S,1),1);
    points = repmat(S(d,:),params.nx+params.nu+1,1)+...
        [zeros(1,params.nx+params.nu);...
        0.1.*eye(params.nx+params.nu)];
    alltest = params.modelfunc(...
        points(:,params.nx+1:end)',points(:,1:params.nx)');
    ftest = alltest(:,idn);
    indices = find(ftest(2:end)==ftest(1));
    if ~isempty(indices)
        lb(:,indices) = -1e-4.*ones(size(lb(:,indices)));
        ub(:,indices) = 1e-4.*ones(size(ub(:,indices)));
    end
    lb1 = lb(1:n1,:);
    lb2 = lb(n1+1:end,:);
    lb = [lb1(:);lb2(:)];
    ub = -lb;
    
    % Optimization ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
    
    for c=1:params.ns
        x0 = randsample(-1:0.01:1,N,true)';
        x0(lb>-1) = 0.*(find(lb>-1));
        try
            [Solutions(c,:),~,res,flag(c),~] = lsqnonlin(...
                @(dv) mmps_lsq(dv,n1,n2,weight,NL,S,nvar),...
                x0,lb,ub,params.lsqopts);
        catch ME
            % nothing
            disp(ME);
        end 
        if res(end)>1
            obj(c) = norm(res(1:end-1),2)./norm(weight,2).*1e6;
        else
            obj(c) = norm(res(1:end-1),2)./norm(weight,2);
        end               
    end

    % Best solution ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~

    [approx.obj,idx] = min(obj);
    approx.OptSol = Solutions(idx,:);
    approx.n1 = n1;
    approx.n2 = n2;
    approx.eltime = toc;

end
